_epochs_training = 25
_train_samples_total_gpu = 32   # batch size across all GPUs
_test_samples_per_gpu = 32      # test batch size
_iters_per_epoch = int(24231 / _train_samples_total_gpu)    # 24231 images in NYUv2.
# _iters_per_epoch = 20    # 24231 images in NYUv2.
_total_iters = int(_epochs_training * _iters_per_epoch)

checkpoint_config = dict(interval=int(_total_iters / 2))
log_config = dict(
    interval=20,
    hooks=[
        dict(type='TextLoggerHook'),
        dict(type='CustomTensorboardLoggerHook')
    ])
custom_hooks = [dict(type='NumClassCheckHook')]

dist_params = dict(backend='nccl')
log_level = 'INFO'
load_from = None
resume_from = None
workflow = [('train', 1)]

num_bins = 2000
num_classes = 80
num_embeddings = 128
num_embeddings_depth = 128
num_vocal = num_bins+1 + num_classes + 2 + num_embeddings + num_embeddings_depth

num_embeddings_others = 128  # innseg uses additional task tokens
num_vocal_insseg = num_bins+1 + num_classes + 2 + num_embeddings + num_embeddings_depth + num_embeddings_others

model = dict(
    type='TwoModelWrapper',
    freeze_model_1=False,
    model_1 = dict(
        type='GenericEncDec',
        decoder_mode="decoder_dl2",
    )
)

img_norm_cfg = dict(
    mean=[123.675, 116.28, 103.53], std=[58.395, 57.12, 57.375], to_rgb=True)

file_client_args = dict(backend='disk')

runner = dict(type='IterBasedRunnerMultitask', max_iters=_total_iters)
evaluation = dict(interval=_total_iters/_epochs_training)

# learning policy

# original
# lr_config = dict(
#     policy='Step',
#     step=[18180],  # 18/25 epoch
#     by_epoch=False,
#     warmup_ratio=0.1,
#     warmup='linear',
#     warmup_by_epoch=False,
#     warmup_iters=500,
# )

# single-step
lr_config = dict(
    policy='OneCycle',
    total_steps=_total_iters,
    max_lr=2e-4,
    by_epoch=False
)

# optimizer
# optimizer = dict(
#     type='AdamW',
#     lr=2e-4,
#     weight_decay=0.05,
#     constructor='SwinLayerDecayOptimizerConstructor',
#     paramwise_cfg=dict(
#         num_layers=[2, 2, 18, 2], layer_decay_rate=0.9,
#         no_decay_names=['relative_position_bias_table', 'rpe_mlp', 'logit_scale',
#                         'det_embed', 'voc_embed', 'enc_embed', 'dec_embed', 'mask_embed'],
#     ))
optimizer = dict(
    type='AdamW',
    lr=2e-4,
    weight_decay=0.05,
    )
# optimizer_config = dict(grad_clip={'max_norm': 10, 'norm_type': 2})
optimizer_config = dict()

task = dict(
    depth=dict(  # len=24231
        times=1,
        data=dict(
            # train=dict(type='nyudepthv2', data_path='data', filenames_path='code/dataset/depth/filenames/',
            #            is_train=True, crop_size=(480, 480), samples_total_gpu=_train_samples_total_gpu),
            # val=dict(type='nyudepthv2', data_path='data', filenames_path='code/dataset/depth/filenames/',
            #          is_train=False, crop_size=(480, 480), samples_per_gpu=_test_samples_per_gpu, workers_per_gpu=8),
            train=dict(type='nyudepthv2', data_path='data', filenames_path='code/dataset/depth/filenames/',
                       is_train=True, crop_size=(480, 640), samples_total_gpu=_train_samples_total_gpu),
            val=dict(type='nyudepthv2', data_path='data', filenames_path='code/dataset/depth/filenames/',
                     is_train=False, crop_size=(480, 640), samples_per_gpu=_test_samples_per_gpu, workers_per_gpu=8, num_images_to_save=5),
        )
    ),
)

# enable fp16
fp16 = dict(loss_scale='dynamic')
